classdef UsbInterface < handle
  % UsbInterface
  %   DEPRECATED, use KeaCamera. Connect to a camera directly using the USB
  %   interface
  %
  % Methods:
  %   isConnected - Check whether a connection is still maintained with the
  %                 camera
  %   testStreamSpeed - Tests the connection speed of the USB port. This is
  %                     useful to check whether a hub or USB port works well.
  %                     The maximum speed that the camera is able to transmit is
  %                     ~3.0 Gbps.
  %   downloadConfiguration - Download camera configuration from the camera
  %   downloadCalibration - Download calibration from the camera
  %   uploadConfiguration - Upload camera configuration to the camera
  %   uploadCalibration - Upload calibration to the camera
  %   depthStreamCapable - Check whether this camera is capable of streaming the
  %                        specified type
  %   startDepthStream - Start the depth stream
  %   stopDepthStream - Stop the depth stream
  %   hasDepthData - Returns true if a depth frame is available
  %   getDepthData - Get a depth frame
  %   imageStreamCapable - Check whether camera supports an image stream
  %   startImageStream - Start image stream
  %   stopImageStream - Stop image stream
  %   hasImageData - Check whether image frame is available
  %   getImageData - Get image frame
  %   getImageDataIntoPointer - Get the image frame into the supplied pointer
  %   getUserPointerCapacity - Get the capacity for amount of user pointers that
  %                            can be registered
  %   setUserPointerCapacity - Set the capacity for amount of user pointers that
  %                            can be registered
  %   addDepthUserPointer - Add a pointer that will be filled with depth data.
  %                         This method allows you to fill data straight into
  %                         your own data structure without any additional
  %                         copying.
  %   version - Get the tof library version running on the camera
  %   softwareTriggerCapable - Check whether the camera is capable of software
  %                            trigger
  %   softwareTrigger - Software trigger the camera
  %

  properties(Access=protected)
    mPtr = uint64(0);
  end

  methods(Access=public)
    function obj = UsbInterface(varargin)
      % UsbInterface
      %   Construct new usb_interface class
      %
      % ----------
      %
      %   Create USB camera from a usb device
      %
      % Inputs:
      %   1 - UsbDevice - USB device
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        obj.mPtr = varargin{1};
      elseif nargin == 1 && isa(varargin{1}, 'tof.UsbDevice')
        obj.mPtr = tof.mex('tof_usb_interface_new', varargin{1}.get_object_ptr());
      else
        error("Can't construct empty object");
      end
    end


    function delete(obj)
      if obj.mPtr ~= 0
        tof.mex('tof_usb_interface_delete', obj.mPtr);
        obj.mPtr = 0;
      end
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function ret_val = isConnected(obj)
      % isConnected
      %   Check whether a connection is still maintained with the camera
      %
      % Output:
      %   ret_val - logical - Is connected
      %

      ret_val = tof.mex('tof_usb_interface_is_connected', obj.mPtr);
    end

    function ret_val = testStreamSpeed(obj)
      % testStreamSpeed
      %   Tests the connection speed of the USB port. This is useful to check
      %   whether a hub or USB port works well. The maximum speed that the
      %   camera is able to transmit is ~3.0 Gbps.
      %
      % Output:
      %   ret_val - numeric - Connection speed in Gpbs (bits)
      %

      ret_val = tof.mex('tof_usb_interface_test_stream_speed', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = downloadConfiguration(obj)
      % downloadConfiguration
      %   Download camera configuration from the camera
      %
      % Output:
      %   ret_val - CameraConfig - Camera configuration
      %

      ret_val = tof.mex('tof_usb_interface_download_configuration', obj.mPtr);
      ret_val = tof.CameraConfig(ret_val);
    end

    function ret_val = downloadCalibration(obj)
      % downloadCalibration
      %   Download calibration from the camera
      %
      % Output:
      %   ret_val - Calibration - Calibration
      %

      ret_val = tof.mex('tof_usb_interface_download_calibration', obj.mPtr);
      ret_val = tof.Calibration(ret_val);
    end

    function uploadConfiguration(obj, config)
      % uploadConfiguration
      %   Upload camera configuration to the camera
      %
      % Inputs:
      %   config - CameraConfig - Camera configuration
      %

      tof.mex('tof_usb_interface_upload_configuration', obj.mPtr, config.get_object_ptr());
    end

    function ret_val = depthStreamCapable(obj, stream)
      % depthStreamCapable
      %   Check whether this camera is capable of streaming the specified type
      %
      % Inputs:
      %   stream - DepthStreamType - Depth stream type
      %
      % Output:
      %   ret_val - logical - Is capable
      %

      ret_val = tof.mex('tof_usb_interface_depth_stream_capable', obj.mPtr, uint32(stream));
    end

    function startDepthStream(obj, stream)
      % startDepthStream
      %   Start the depth stream
      %
      % Inputs:
      %   stream - DepthStreamType - Depth stream type
      %

      tof.mex('tof_usb_interface_start_depth_stream', obj.mPtr, uint32(stream));
    end

    function stopDepthStream(obj)
      % stopDepthStream
      %   Stop the depth stream
      %

      tof.mex('tof_usb_interface_stop_depth_stream', obj.mPtr);
    end

    function ret_val = hasDepthData(obj)
      % hasDepthData
      %   Returns true if a depth frame is available
      %
      % Output:
      %   ret_val - logical - Depth frame available
      %

      ret_val = tof.mex('tof_usb_interface_has_depth_data', obj.mPtr);
    end

    function ret_val = getDepthData(obj)
      % getDepthData
      %   Get a depth frame
      %
      % Output:
      %   ret_val - Data - Depth frame
      %

      ret_val = tof.mex('tof_usb_interface_get_depth_data', obj.mPtr);
      ret_val = tof.Data(ret_val);
    end

    function ret_val = imageStreamCapable(obj)
      % imageStreamCapable
      %   Check whether camera supports an image stream
      %
      % Output:
      %   ret_val - logical - Camera supports an image stream
      %

      ret_val = tof.mex('tof_usb_interface_image_stream_capable', obj.mPtr);
    end

    function startImageStream(obj)
      % startImageStream
      %   Start image stream
      %

      tof.mex('tof_usb_interface_start_image_stream', obj.mPtr);
    end

    function stopImageStream(obj)
      % stopImageStream
      %   Stop image stream
      %

      tof.mex('tof_usb_interface_stop_image_stream', obj.mPtr);
    end

    function ret_val = hasImageData(obj)
      % hasImageData
      %   Check whether image frame is available
      %
      % Output:
      %   ret_val - logical - Image frame available
      %

      ret_val = tof.mex('tof_usb_interface_has_image_data', obj.mPtr);
    end

    function ret_val = getImageData(obj)
      % getImageData
      %   Get image frame
      %
      % Output:
      %   ret_val - Data - Image data
      %

      ret_val = tof.mex('tof_usb_interface_get_image_data', obj.mPtr);
      ret_val = tof.Data(ret_val);
    end

    function ret_val = getUserPointerCapacity(obj)
      % getUserPointerCapacity
      %   Get the capacity for amount of user pointers that can be registered
      %
      % Output:
      %   ret_val - numeric - User pointer capacity
      %

      ret_val = tof.mex('tof_usb_interface_get_user_pointer_capacity', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setUserPointerCapacity(obj, capacity)
      % setUserPointerCapacity
      %   Set the capacity for amount of user pointers that can be registered
      %
      % Inputs:
      %   capacity - numeric - User pointer capacity
      %

      tof.mex('tof_usb_interface_set_user_pointer_capacity', obj.mPtr, capacity);
    end

    function ret_val = version(obj)
      % version
      %   Get the tof library version running on the camera
      %
      % Output:
      %   ret_val - string - Version
      %

      ret_val = tof.mex('tof_usb_interface_version', obj.mPtr);
    end

    function ret_val = softwareTriggerCapable(obj)
      % softwareTriggerCapable
      %   Check whether the camera is capable of software trigger
      %
      % Output:
      %   ret_val - logical - Software trigger capable
      %

      ret_val = tof.mex('tof_usb_interface_software_trigger_capable', obj.mPtr);
    end

    function softwareTrigger(obj)
      % softwareTrigger
      %   Software trigger the camera
      %

      tof.mex('tof_usb_interface_software_trigger', obj.mPtr);
    end

  end
end
